<?php

/**
 * DRD Server Action to find out all domains on thsi Drupal installation.
 *
 * @return string
 */
function drd_server_server_domains() {
  $sites = drd_server_read_sites();
  $output = array();
  $error = array();
  foreach ($sites as $url => $id) {
    if (isset($output[$id])) {
      $output[$id]['aliases'][] = $url;
    }
    else if (isset($error[$id])) {
      // Ignore this alias, it won't work
    }
    else {
      $code = drd_server_server_check_host($url, $id);
      if ($code === FALSE) {
        $error[$id] = TRUE;
      }
      else {
        $output[$id] = array(
          'url' => $url,
          'aliases' => array(),
          'drd installed' => ($code === TRUE),
        );
      }
    }
  }
  return drd_server_result('server.domains', $output);
}

/**
 * Function called by drd_server_server_domains() to check a specific URL
 * if it really exists and if the module drd_server is installed.
 *
 * @param string $url
 * @param string $dir
 * @return int|bool
 */
function drd_server_server_check_host($url, $dir) {
  $id = rand();
  try {
    $check_url = 'http://' . $url;
    $check = drupal_http_request($check_url . '/?q=admin/drd_server/' . $id);
    $ok = (($check->code == 200) || (in_array($check->code, array(301, 302, 307)) && $check->redirect_code == 200));
    if (!$ok) {
      // Maybe the url is running https only, so we try that as well.
      $check_url = 'https://' . $url;
      $check = drupal_http_request($check_url . '/?q=admin/drd_server/' . $id);
      $ok = (($check->code == 200) || (in_array($check->code, array(301, 302, 307)) && $check->redirect_code == 200));
    }
    if ($ok && $check->data == _drd_server_get_check_token($check_url, $id)) {
      return TRUE;
    }
    return $check->code;
  }
  catch (Exception $e) {
    return FALSE;
  }
}

/**
 * DRD Server Action to retrieve the PHP error log from the OS.
 *
 * @param int $max_length
 * @return string
 */
function drd_server_server_php_error_log($max_length = 5000) {
  $log_file = ini_get('error_log');
  if (!file_exists($log_file)) {
    return drd_server_error(t('PHP Error Log does not exist.'), DRD_ERROR_NO_LOG_PHP);
  }
  $offset = max(-1, (filesize($log_file) - $max_length));
  $log = file_get_contents($log_file, FILE_BINARY, NULL, $offset);
  $result['cache']['php_error_log']['server'] = array(
    'title' => 'PHP Error Log',
    'callback' => 'drupal_render',
    'arguments' => array(
      'data' => array(
        '#markup' => '<pre>' . $log . '</pre>',
      ),
    ),
  );
  return drd_server_result('server.php.error.log', $result);
}
